import java.io.*;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.Iterator;
import java.util.Random;


public class Read {
	public static class BidData{
		public HashSet<Integer> region;
		public Integer company;
		public Double value;	
		public String toString(){
			String s=new String();
			s="Company:"+company.toString();
			s=s+" Bid Amount:"+value.toString();
			s=s+" Regions:"+region.toString();
			return s;
		}
	}
	/* Read Bids from File, Store in Bids */
	public static Boolean ReadFile(String fname,HashMap<Integer,BidData> Bids){
		 try{
			 FileReader input = new FileReader(fname);
			 BufferedReader bufRead = new BufferedReader(input);
			 String line;   
			 line = bufRead.readLine();
			 int count = 0;   
			 // Reading lines ending with a # (bids), and store in Bids
			
			 while (line != null){			  
				 if(line.endsWith("#")){
				  String[] res=line.split(" ");
				  BidData curbid= new BidData();
				  curbid.company=Integer.parseInt(res[0]);
				  curbid.value=Double.parseDouble(res[1]);
				  curbid.region=new HashSet<Integer>();
				  for(int k=2;k<res.length-1;k++){
						  curbid.region.add(Integer.parseInt(res[k]));					  
				  }		
				  Bids.put(count,curbid);
				  count++;			      				  
				 }
				 line = bufRead.readLine();		      
			 }
			 bufRead.close();
			 return true;
		 }
		 catch(IOException e){
			 e.printStackTrace();	
			 return false;
		 }
	}
	
	/*Generate Random Feasible Solution For Problem */
	public static double GenRandom(HashMap<Integer,BidData> Bids,LinkedList<Integer> accepted){
		  //Set<Integer> Avail=(Bids.keySet());		
		  HashSet<Integer> Avail=new HashSet<Integer>(Bids.keySet());
		  Random gen=new Random();
		  Double value=0.0;
		  while(Avail.size()>0){
			  Integer newbid=gen.nextInt(Avail.size());
			  newbid=(Integer)Avail.toArray()[newbid];
			  value+=Bids.get(newbid).value;					  
			  accepted.add(newbid);
			  HashSet<Integer> Remove=new HashSet<Integer>();
			  Iterator<Integer> it=Avail.iterator();
			  while(it.hasNext()){
				  Integer k=it.next();
				  HashSet<Integer> intersection = new HashSet<Integer>(Bids.get(k).region);
				  intersection.retainAll(Bids.get(newbid).region);
				  if((intersection.size()>0) || (Bids.get(newbid).company.equals(Bids.get(k).company))){
					  Remove.add(k);
				  }
			  }
			  Avail.removeAll(Remove);
		  }
		  Collections.sort(accepted);
		  return value;
	}
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		try{		  	    
		  
		  /*Data Structures to Store Bids*/
		  HashMap<Integer,BidData> Bids=new HashMap<Integer,BidData>();
	    /*Read Bids from File */  
		  ReadFile(args[0],Bids);
		  
		  /*Generate Random Solution*/
		  LinkedList<Integer> accepted=new LinkedList<Integer>();
		  Double value=GenRandom(Bids,accepted);		  		
			
		  /*Print Solution and Total Revenue*/
		  System.out.println("Value : "+value.toString());
		  
		  Iterator<Integer> it=accepted.iterator();
		  System.out.print("Solution : ");
		  while(it.hasNext()){
			  Integer i=it.next();
			  System.out.print(i.toString()+" ");			  
		  }
		  System.out.print("#\n");
		  
		  }catch (ArrayIndexOutOfBoundsException e){		             
			  System.out.println("Usage: java ReadFile filename\n");          
		  }		  
	}

}
